@@ -135,6 +135,7 @@ CVisGUIShader *vis_shader = NULL;
 
 
 using namespace ADDON;
+using namespace PLATFORM;
 
 
 class cOSDTexture
@@ -360,10 +361,10 @@ class cOSDRenderGL : public cOSDRender
 public:
   cOSDRenderGL();
   virtual ~cOSDRenderGL();
-  virtual void DisposeTexture(int wndId);
-  virtual void FreeResources();
-  virtual void Render();
-  virtual bool Init();
+  void DisposeTexture(int wndId) override;
+  void FreeResources() override;
+  void Render() override;
+  bool Init() override;
 protected:
   GLuint m_hwTextures[MAX_TEXTURES];
   std::queue<GLuint> m_disposedHwTextures;
@@ -396,12 +397,10 @@ bool cOSDRenderGL::Init()
 #if defined(HAVE_GLES2)
   vis_shader = new CVisGUIShader(vert, frag);
 
-  if(!vis_shader)
-    return false;
-
   if(!vis_shader->CompileAndLink())
   {
     delete vis_shader;
+    vis_shader = NULL;
     return false;
   }
 #endif
@@ -610,10 +609,10 @@ class cOSDRenderDX : public cOSDRender
 public:
   cOSDRenderDX();
   virtual ~cOSDRenderDX();
-  virtual void DisposeTexture(int wndId);
-  virtual void FreeResources();
-  virtual void Render();
-  virtual void SetDevice(void *device) { m_device = (LPDIRECT3DDEVICE9)device; };
+  void DisposeTexture(int wndId) override;
+  void FreeResources() override;
+  void Render() override;
+  void SetDevice(void *device) override { m_device = (LPDIRECT3DDEVICE9)device; };
 protected:
   LPDIRECT3DDEVICE9 m_device;
   LPDIRECT3DTEXTURE9 m_hwTextures[MAX_TEXTURES];
@@ -837,6 +836,7 @@ bool cVNSIAdmin::Open(const std::string& hostname, int port, const char* name)
   if(!m_osdRender->Init())
   {
     delete m_osdRender;
+    m_osdRender = NULL;
     return false;
   }
 
@@ -871,6 +871,7 @@ bool cVNSIAdmin::Open(const std::string& hostname, int port, const char* name)
   if (m_osdRender)
   {
     delete m_osdRender;
+    m_osdRender = NULL;
   }
 
   return true;
@@ -882,10 +883,10 @@ bool cVNSIAdmin::OnClick(int controlId)
   {
     int value = m_spinTimeshiftMode->GetValue();
     cRequestPacket vrp;
-    if (!vrp.init(VNSI_STORESETUP) ||
-        !vrp.add_String(CONFNAME_TIMESHIFT) ||
-        !vrp.add_U32(value) ||
-        ReadSuccess(&vrp))
+    vrp.init(VNSI_STORESETUP);
+    vrp.add_String(CONFNAME_TIMESHIFT);
+    vrp.add_U32(value);
+    if (!ReadSuccess(&vrp))
     {
       XBMC->Log(LOG_ERROR, "%s - failed to set timeshift mode", __FUNCTION__);
     }
@@ -895,10 +896,10 @@ bool cVNSIAdmin::OnClick(int controlId)
   {
     int value = m_spinTimeshiftBufferRam->GetValue();
     cRequestPacket vrp;
-    if (!vrp.init(VNSI_STORESETUP) ||
-        !vrp.add_String(CONFNAME_TIMESHIFTBUFFERSIZE) ||
-        !vrp.add_U32(value) ||
-        ReadSuccess(&vrp))
+    vrp.init(VNSI_STORESETUP);
+    vrp.add_String(CONFNAME_TIMESHIFTBUFFERSIZE);
+    vrp.add_U32(value);
+    if (!ReadSuccess(&vrp))
     {
       XBMC->Log(LOG_ERROR, "%s - failed to set timeshift buffer", __FUNCTION__);
     }
@@ -908,10 +909,10 @@ bool cVNSIAdmin::OnClick(int controlId)
   {
     int value = m_spinTimeshiftBufferFile->GetValue();
     cRequestPacket vrp;
-    if (!vrp.init(VNSI_STORESETUP) ||
-        !vrp.add_String(CONFNAME_TIMESHIFTBUFFERFILESIZE) ||
-        !vrp.add_U32(value) ||
-        ReadSuccess(&vrp))
+    vrp.init(VNSI_STORESETUP);
+    vrp.add_String(CONFNAME_TIMESHIFTBUFFERFILESIZE);
+    vrp.add_U32(value);
+    if (!ReadSuccess(&vrp))
     {
       XBMC->Log(LOG_ERROR, "%s - failed to set timeshift buffer file", __FUNCTION__);
     }
@@ -1036,11 +1037,7 @@ bool cVNSIAdmin::OnInit()
   m_renderControl->Init();
 
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_OSD_HITKEY))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return false;
-  }
+  vrp.init(VNSI_OSD_HITKEY);
   vrp.add_U32(0);
   cVNSISession::TransmitMessage(&vrp);
 #endif
@@ -1054,12 +1051,9 @@ bool cVNSIAdmin::OnInit()
 
   {
     cRequestPacket vrp;
-    if (!vrp.init(VNSI_GETSETUP) || !vrp.add_String(CONFNAME_TIMESHIFT))
-    {
-      XBMC->Log(LOG_ERROR, "%s - failed to get timeshift mode", __FUNCTION__);
-      return false;
-    }
-    cResponsePacket *resp = ReadResult(&vrp);
+    vrp.init(VNSI_GETSETUP);
+    vrp.add_String(CONFNAME_TIMESHIFT);
+    auto resp = ReadResult(&vrp);
     if (!resp)
     {
       XBMC->Log(LOG_ERROR, "%s - failed to get timeshift mode", __FUNCTION__);
@@ -1067,7 +1061,6 @@ bool cVNSIAdmin::OnInit()
     }
     int mode = resp->extract_U32();
     m_spinTimeshiftMode->SetValue(mode);
-    delete resp;
   }
 
   m_spinTimeshiftBufferRam = GUI->Control_getSpin(m_window, CONTROL_SPIN_TIMESHIFT_BUFFER_RAM);
@@ -1081,12 +1074,9 @@ bool cVNSIAdmin::OnInit()
 
   {
     cRequestPacket vrp;
-    if (!vrp.init(VNSI_GETSETUP) || !vrp.add_String(CONFNAME_TIMESHIFTBUFFERSIZE))
-    {
-      XBMC->Log(LOG_ERROR, "%s - failed to get timeshift buffer size", __FUNCTION__);
-      return false;
-    }
-    cResponsePacket *resp = ReadResult(&vrp);
+    vrp.init(VNSI_GETSETUP);
+    vrp.add_String(CONFNAME_TIMESHIFTBUFFERSIZE);
+    auto resp = ReadResult(&vrp);
     if (!resp)
     {
       XBMC->Log(LOG_ERROR, "%s - failed to get timeshift buffer size", __FUNCTION__);
@@ -1094,7 +1084,6 @@ bool cVNSIAdmin::OnInit()
     }
     int mode = resp->extract_U32();
     m_spinTimeshiftBufferRam->SetValue(mode);
-    delete resp;
   }
   m_spinTimeshiftBufferFile = GUI->Control_getSpin(m_window, CONTROL_SPIN_TIMESHIFT_BUFFER_FILE);
   m_spinTimeshiftBufferFile->Clear();
@@ -1106,12 +1095,9 @@ bool cVNSIAdmin::OnInit()
 
   {
     cRequestPacket vrp;
-    if (!vrp.init(VNSI_GETSETUP) || !vrp.add_String(CONFNAME_TIMESHIFTBUFFERFILESIZE))
-    {
-      XBMC->Log(LOG_ERROR, "%s - failed to get timeshift buffer (file) size", __FUNCTION__);
-      return false;
-    }
-    cResponsePacket *resp = ReadResult(&vrp);
+    vrp.init(VNSI_GETSETUP);
+    vrp.add_String(CONFNAME_TIMESHIFTBUFFERFILESIZE);
+    auto resp = ReadResult(&vrp);
     if (!resp)
     {
       XBMC->Log(LOG_ERROR, "%s - failed to get timeshift buffer (file) size", __FUNCTION__);
@@ -1119,7 +1105,6 @@ bool cVNSIAdmin::OnInit()
     }
     int mode = resp->extract_U32();
     m_spinTimeshiftBufferFile->SetValue(mode);
-    delete resp;
   }
 
   // channel filters
@@ -1149,11 +1134,7 @@ bool cVNSIAdmin::OnAction(int actionId)
     {
       // send all actions to vdr
       cRequestPacket vrp;
-      if (!vrp.init(VNSI_OSD_HITKEY))
-      {
-        XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-        return false;
-      }
+      vrp.init(VNSI_OSD_HITKEY);
       vrp.add_U32(actionId);
       cVNSISession::TransmitMessage(&vrp);
       return true;
@@ -1226,25 +1207,23 @@ bool cVNSIAdmin::Create(int x, int y, int w, int h, void* device)
 
 void cVNSIAdmin::Render()
 {
-  m_osdMutex.Lock();
+  const CLockObject lock(m_osdMutex);
   if (m_osdRender)
   {
     m_osdRender->Render();
     m_osdRender->FreeResources();
   }
   m_bIsOsdDirty = false;
-  m_osdMutex.Unlock();
 }
 
 void cVNSIAdmin::Stop()
 {
-  m_osdMutex.Lock();
+  const CLockObject lock(m_osdMutex);
   if (m_osdRender)
   {
     delete m_osdRender;
     m_osdRender = NULL;
   }
-  m_osdMutex.Unlock();
 }
 
 bool cVNSIAdmin::Dirty()
@@ -1255,13 +1234,23 @@ bool cVNSIAdmin::Dirty()
 bool cVNSIAdmin::OnInitCB(GUIHANDLE cbhdl)
 {
   cVNSIAdmin* osd = static_cast<cVNSIAdmin*>(cbhdl);
-  return osd->OnInit();
+  try {
+    return osd->OnInit();
+  } catch (std::exception e) {
+    XBMC->Log(LOG_ERROR, "%s - %s", __FUNCTION__, e.what());
+    return false;
+  }
 }
 
 bool cVNSIAdmin::OnClickCB(GUIHANDLE cbhdl, int controlId)
 {
   cVNSIAdmin* osd = static_cast<cVNSIAdmin*>(cbhdl);
-  return osd->OnClick(controlId);
+  try {
+    return osd->OnClick(controlId);
+  } catch (std::exception e) {
+    XBMC->Log(LOG_ERROR, "%s - %s", __FUNCTION__, e.what());
+    return false;
+  }
 }
 
 bool cVNSIAdmin::OnFocusCB(GUIHANDLE cbhdl, int controlId)
@@ -1273,7 +1262,12 @@ bool cVNSIAdmin::OnFocusCB(GUIHANDLE cbhdl, int controlId)
 bool cVNSIAdmin::OnActionCB(GUIHANDLE cbhdl, int actionId)
 {
   cVNSIAdmin* osd = static_cast<cVNSIAdmin*>(cbhdl);
-  return osd->OnAction(actionId);
+  try {
+    return osd->OnAction(actionId);
+  } catch (std::exception e) {
+    XBMC->Log(LOG_ERROR, "%s - %s", __FUNCTION__, e.what());
+    return false;
+  }
 }
 
 bool cVNSIAdmin::CreateCB(GUIHANDLE cbhdl, int x, int y, int w, int h, void *device)
@@ -1316,50 +1310,44 @@ bool cVNSIAdmin::OnResponsePacket(cResponsePacket* resp)
     {
       data = resp->getUserData();
       len = resp->getUserDataLength();
-      m_osdMutex.Lock();
+      const CLockObject lock(m_osdMutex);
       if (m_osdRender)
         m_osdRender->AddTexture(wnd, color, x0, y0, x1, y1, data[0]);
-      m_osdMutex.Unlock();
-      free(data);
     }
     else if (resp->getOpCodeID() == VNSI_OSD_SETPALETTE)
     {
       data = resp->getUserData();
       len = resp->getUserDataLength();
-      m_osdMutex.Lock();
+      const CLockObject lock(m_osdMutex);
       if (m_osdRender)
         m_osdRender->SetPalette(wnd, x0, (uint32_t*)data);
-      m_osdMutex.Unlock();
-      free(data);
     }
     else if (resp->getOpCodeID() == VNSI_OSD_SETBLOCK)
     {
       data = resp->getUserData();
       len = resp->getUserDataLength();
-      m_osdMutex.Lock();
+      const CLockObject lock(m_osdMutex);
       if (m_osdRender)
       {
         m_osdRender->SetBlock(wnd, x0, y0, x1, y1, color, data, len);
         m_bIsOsdDirty = true;
       }
-      m_osdMutex.Unlock();
-      free(data);
     }
     else if (resp->getOpCodeID() == VNSI_OSD_CLEAR)
     {
-      m_osdMutex.Lock();
+      const CLockObject lock(m_osdMutex);
       if (m_osdRender)
         m_osdRender->Clear(wnd);
       m_bIsOsdDirty = true;
-      m_osdMutex.Unlock();
     }
     else if (resp->getOpCodeID() == VNSI_OSD_CLOSE)
     {
-      m_osdMutex.Lock();
-      if (m_osdRender)
-        m_osdRender->DisposeTexture(wnd);
-      m_bIsOsdDirty = true;
-      m_osdMutex.Unlock();
+      {
+        const CLockObject lock(m_osdMutex);
+        if (m_osdRender)
+          m_osdRender->DisposeTexture(wnd);
+        m_bIsOsdDirty = true;
+      }
       m_window->SetFocusId(CONTROL_MENU);
     }
     else if (resp->getOpCodeID() == VNSI_OSD_MOVEWINDOW)
@@ -1377,20 +1365,17 @@ bool cVNSIAdmin::OnResponsePacket(cResponsePacket* resp)
 bool cVNSIAdmin::ConnectOSD()
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_OSD_CONNECT))
-    return false;
+  vrp.init(VNSI_OSD_CONNECT);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (vresp == NULL || vresp->noResponse())
   {
-    delete vresp;
     return false;
   }
   uint32_t osdWidth = vresp->extract_U32();
   uint32_t osdHeight = vresp->extract_U32();
   if (m_osdRender)
     m_osdRender->SetOSDSize(osdWidth, osdHeight);
-  delete vresp;
 
   return true;
 }
@@ -1398,23 +1383,11 @@ bool cVNSIAdmin::ConnectOSD()
 bool cVNSIAdmin::ReadChannelList(bool radio)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_CHANNELS_GETCHANNELS))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return false;
-  }
-  if (!vrp.add_U32(radio))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't add parameter to cRequestPacket", __FUNCTION__);
-    return false;
-  }
-  if (!vrp.add_U8(0)) // apply no filter
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't add parameter to cRequestPacket", __FUNCTION__);
-    return false;
-  }
+  vrp.init(VNSI_CHANNELS_GETCHANNELS);
+  vrp.add_U32(radio);
+  vrp.add_U8(0); // apply no filter
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
@@ -1423,7 +1396,7 @@ bool cVNSIAdmin::ReadChannelList(bool radio)
 
   m_channels.m_channels.clear();
   m_channels.m_channelsMap.clear();
-  while (!vresp->end())
+  while (vresp->getRemainingLength() >= 3 * 4 + 3)
   {
     CChannel channel;
     channel.m_blacklist = false;
@@ -1443,13 +1416,9 @@ bool cVNSIAdmin::ReadChannelList(bool radio)
     }
     channel.m_radio       = radio;
 
-    delete[] strChannelName;
-    delete[] strProviderName;
-    delete[] strCaids;
     m_channels.m_channels.push_back(channel);
     m_channels.m_channelsMap[channel.m_id] = m_channels.m_channels.size() - 1;
   }
-  delete vresp;
 
   return true;
 }
@@ -1457,18 +1426,10 @@ bool cVNSIAdmin::ReadChannelList(bool radio)
 bool cVNSIAdmin::ReadChannelWhitelist(bool radio)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_CHANNELS_GETWHITELIST))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return false;
-  }
-  if (!vrp.add_U8(radio))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't add parameter to cRequestPacket", __FUNCTION__);
-    return false;
-  }
+  vrp.init(VNSI_CHANNELS_GETWHITELIST);
+  vrp.add_U8(radio);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
@@ -1477,15 +1438,13 @@ bool cVNSIAdmin::ReadChannelWhitelist(bool radio)
 
   m_channels.m_providerWhitelist.clear();
   CProvider provider;
-  while (!vresp->end())
+  while (vresp->getRemainingLength() >= 1 + 4)
   {
     char *strProviderName = vresp->extract_String();
     provider.m_name = strProviderName;
     provider.m_caid = vresp->extract_U32();
     m_channels.m_providerWhitelist.push_back(provider);
-    delete [] strProviderName;
   }
-  delete vresp;
 
   return true;
 }
@@ -1495,24 +1454,16 @@ bool cVNSIAdmin::SaveChannelWhitelist(bool radio)
   m_channels.ExtractProviderWhitelist();
 
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_CHANNELS_SETWHITELIST))
+  vrp.init(VNSI_CHANNELS_SETWHITELIST);
+  vrp.add_U8(radio);
+
+  for (const auto &provider : m_channels.m_providerWhitelist)
   {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return false;
-  }
-  if (!vrp.add_U8(radio))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't add parameter to cRequestPacket", __FUNCTION__);
-    return false;
+    vrp.add_String(provider.m_name.c_str());
+    vrp.add_S32(provider.m_caid);
   }
 
-  for(unsigned int i=0; i<m_channels.m_providerWhitelist.size(); i++)
-  {
-    vrp.add_String(m_channels.m_providerWhitelist[i].m_name.c_str());
-    vrp.add_S32(m_channels.m_providerWhitelist[i].m_caid);
-  }
-
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
@@ -1525,18 +1476,10 @@ bool cVNSIAdmin::SaveChannelWhitelist(bool radio)
 bool cVNSIAdmin::ReadChannelBlacklist(bool radio)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_CHANNELS_GETBLACKLIST))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return false;
-  }
-  if (!vrp.add_U8(radio))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't add parameter to cRequestPacket", __FUNCTION__);
-    return false;
-  }
+  vrp.init(VNSI_CHANNELS_GETBLACKLIST);
+  vrp.add_U8(radio);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
@@ -1544,12 +1487,11 @@ bool cVNSIAdmin::ReadChannelBlacklist(bool radio)
   }
 
   m_channels.m_channelBlacklist.clear();
-  while (!vresp->end())
+  while (vresp->getRemainingLength() >= 4)
   {
     int id = vresp->extract_U32();
     m_channels.m_channelBlacklist.push_back(id);
   }
-  delete vresp;
 
   return true;
 }
@@ -1559,23 +1501,15 @@ bool cVNSIAdmin::SaveChannelBlacklist(bool radio)
   m_channels.ExtractChannelBlacklist();
 
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_CHANNELS_SETBLACKLIST))
+  vrp.init(VNSI_CHANNELS_SETBLACKLIST);
+  vrp.add_U8(radio);
+
+  for (auto b : m_channels.m_channelBlacklist)
   {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return false;
-  }
-  if (!vrp.add_U8(radio))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't add parameter to cRequestPacket", __FUNCTION__);
-    return false;
+    vrp.add_S32(b);
   }
 
-  for(unsigned int i=0; i<m_channels.m_channelBlacklist.size(); i++)
-  {
-    vrp.add_S32(m_channels.m_channelBlacklist[i]);
-  }
-
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
@@ -1588,10 +1522,9 @@ bool cVNSIAdmin::SaveChannelBlacklist(bool radio)
 void cVNSIAdmin::ClearListItems()
 {
   m_window->ClearList();
-  std::vector<CAddonListItem*>::iterator it;
-  for(it=m_listItems.begin(); it!=m_listItems.end(); ++it)
+  for (auto *i : m_listItems)
   {
-    GUI->ListItem_destroy(*it);
+    GUI->ListItem_destroy(i);
   }
   m_listItems.clear();
   m_listItemsMap.clear();
@@ -1602,16 +1535,15 @@ void cVNSIAdmin::LoadListItemsProviders()
 {
   ClearListItems();
 
-  std::vector<CProvider>::iterator it;
   int count = 0;
-  for(it=m_channels.m_providers.begin(); it!=m_channels.m_providers.end(); ++it)
+  for (const auto &provider : m_channels.m_providers)
   {
     std::string tmp;
-    if(!it->m_name.empty())
-      tmp = it->m_name;
+    if(!provider.m_name.empty())
+      tmp = provider.m_name;
     else
       tmp = XBMC->GetLocalizedString(30114);
-    if (it->m_caid == 0)
+    if (provider.m_caid == 0)
     {
       tmp += " - FTA";
     }
@@ -1619,7 +1551,7 @@ void cVNSIAdmin::LoadListItemsProviders()
     {
       tmp += " - CAID: ";
       char buf[16];
-      sprintf(buf, "%04x", it->m_caid);
+      sprintf(buf, "%04x", provider.m_caid);
       tmp += buf;
     }
 
@@ -1629,7 +1561,7 @@ void cVNSIAdmin::LoadListItemsProviders()
     m_listItems.push_back(item);
     m_listItemsMap[hdl] = count;
 
-    if (it->m_whitelist)
+    if (provider.m_whitelist)
       item->SetProperty("IsWhitelist", "true");
     else
       item->SetProperty("IsWhitelist", "false");
