@@ -28,6 +28,33 @@
 using namespace ADDON;
 using namespace PLATFORM;
 
+cVNSIData::SMessage &
+cVNSIData::Queue::Enqueue(uint32_t serial)
+{
+  const CLockObject lock(m_mutex);
+  return m_queue[serial];
+}
+
+std::unique_ptr<cResponsePacket>
+cVNSIData::Queue::Dequeue(uint32_t serial, SMessage &message)
+{
+  const CLockObject lock(m_mutex);
+  auto vresp = std::move(message.pkt);
+  m_queue.erase(serial);
+  return vresp;
+}
+
+void
+cVNSIData::Queue::Set(std::unique_ptr<cResponsePacket> &&vresp)
+{
+  CLockObject lock(m_mutex);
+  SMessages::iterator it = m_queue.find(vresp->getRequestID());
+  if (it != m_queue.end()) {
+    it->second.pkt = std::move(vresp);
+    it->second.event.Broadcast();
+  }
+}
+
 cVNSIData::cVNSIData()
 {
 }
@@ -83,49 +110,25 @@ void cVNSIData::OnReconnect()
   PVR->TriggerRecordingUpdate();
 }
 
-cResponsePacket* cVNSIData::ReadResult(cRequestPacket* vrp)
+std::unique_ptr<cResponsePacket> cVNSIData::ReadResult(cRequestPacket* vrp)
 {
-  m_mutex.Lock();
+  SMessage &message = m_queue.Enqueue(vrp->getSerial());
 
-  SMessage &message(m_queue[vrp->getSerial()]);
-  message.event = new CEvent;
-  message.pkt   = NULL;
-
-  m_mutex.Unlock();
-
-  if(!cVNSISession::TransmitMessage(vrp))
-  {
-    m_queue.erase(vrp->getSerial());
-    return NULL;
-  }
-
-  if (!message.event->Wait(g_iConnectTimeout * 1000))
+  if (cVNSISession::TransmitMessage(vrp) &&
+      !message.event.Wait(g_iConnectTimeout * 1000))
   {
     XBMC->Log(LOG_ERROR, "%s - request timed out after %d seconds", __FUNCTION__, g_iConnectTimeout);
   }
 
-  m_mutex.Lock();
-
-  cResponsePacket* vresp = message.pkt;
-  delete message.event;
-
-  m_queue.erase(vrp->getSerial());
-
-  m_mutex.Unlock();
-
-  return vresp;
+  return m_queue.Dequeue(vrp->getSerial(), message);
 }
 
 bool cVNSIData::GetDriveSpace(long long *total, long long *used)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_RECORDINGS_DISKSIZE))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return false;
-  }
+  vrp.init(VNSI_RECORDINGS_DISKSIZE);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
@@ -143,20 +146,15 @@ bool cVNSIData::GetDriveSpace(long long *total, long long *used)
   *total *= 1024;
   *used  *= 1024;
 
-  delete vresp;
   return true;
 }
 
 bool cVNSIData::SupportChannelScan()
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_SCAN_SUPPORTED))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return false;
-  }
+  vrp.init(VNSI_SCAN_SUPPORTED);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
@@ -164,7 +162,6 @@ bool cVNSIData::SupportChannelScan()
   }
 
   uint32_t ret = vresp->extract_U32();
-  delete vresp;
   return ret == VNSI_RET_OK ? true : false;
 }
 
@@ -173,13 +170,9 @@ bool cVNSIData::SupportRecordingsUndelete()
   if (GetProtocol() > 7)
   {
     cRequestPacket vrp;
-    if (!vrp.init(VNSI_RECORDINGS_DELETED_ACCESS_SUPPORTED))
-    {
-      XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-      return false;
-    }
+    vrp.init(VNSI_RECORDINGS_DELETED_ACCESS_SUPPORTED);
 
-    cResponsePacket* vresp = ReadResult(&vrp);
+    auto vresp = ReadResult(&vrp);
     if (!vresp)
     {
       XBMC->Log(LOG_INFO, "%s - Can't get response packed", __FUNCTION__);
@@ -187,7 +180,6 @@ bool cVNSIData::SupportRecordingsUndelete()
     }
 
     uint32_t ret = vresp->extract_U32();
-    delete vresp;
     return ret == VNSI_RET_OK ? true : false;
   }
 
@@ -198,10 +190,10 @@ bool cVNSIData::SupportRecordingsUndelete()
 bool cVNSIData::EnableStatusInterface(bool onOff)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_ENABLESTATUSINTERFACE)) return false;
-  if (!vrp.add_U8(onOff)) return false;
+  vrp.init(VNSI_ENABLESTATUSINTERFACE);
+  vrp.add_U8(onOff);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
@@ -209,20 +201,15 @@ bool cVNSIData::EnableStatusInterface(bool onOff)
   }
 
   uint32_t ret = vresp->extract_U32();
-  delete vresp;
   return ret == VNSI_RET_OK ? true : false;
 }
 
 int cVNSIData::GetChannelsCount()
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_CHANNELS_GETCOUNT))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return -1;
-  }
+  vrp.init(VNSI_CHANNELS_GETCOUNT);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
@@ -230,38 +217,24 @@ int cVNSIData::GetChannelsCount()
   }
 
   uint32_t count = vresp->extract_U32();
-
-  delete vresp;
   return count;
 }
 
 bool cVNSIData::GetChannelsList(ADDON_HANDLE handle, bool radio)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_CHANNELS_GETCHANNELS))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return false;
-  }
-  if (!vrp.add_U32(radio))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't add parameter to cRequestPacket", __FUNCTION__);
-    return false;
-  }
-  if (!vrp.add_U8(1)) // apply filter
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't add parameter to cRequestPacket", __FUNCTION__);
-    return false;
-  }
+  vrp.init(VNSI_CHANNELS_GETCHANNELS);
+  vrp.add_U32(radio);
+  vrp.add_U8(1); // apply filter
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
     return false;
   }
 
-  while (!vresp->end())
+  while (vresp->getRemainingLength() >= 3 * 4 + 3)
   {
     PVR_CHANNEL tag;
     memset(&tag, 0 , sizeof(tag));
@@ -289,37 +262,27 @@ bool cVNSIData::GetChannelsList(ADDON_HANDLE handle, bool radio)
     tag.bIsRadio          = radio;
 
     PVR->TransferChannelEntry(handle, &tag);
-    delete[] strChannelName;
-    delete[] strProviderName;
-    delete[] strCaids;
   }
 
-  delete vresp;
   return true;
 }
 
 bool cVNSIData::GetEPGForChannel(ADDON_HANDLE handle, const PVR_CHANNEL &channel, time_t start, time_t end)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_EPG_GETFORCHANNEL))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return false;
-  }
-  if (!vrp.add_U32(channel.iUniqueId) || !vrp.add_U32(start) || !vrp.add_U32(end - start))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't add parameter to cRequestPacket", __FUNCTION__);
-    return false;
-  }
+  vrp.init(VNSI_EPG_GETFORCHANNEL);
+  vrp.add_U32(channel.iUniqueId);
+  vrp.add_U32(start);
+  vrp.add_U32(end - start);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
     return false;
   }
 
-  while (!vresp->end())
+  while (vresp->getRemainingLength() >= 5 * 4 + 3)
   {
     EPG_TAG tag;
     memset(&tag, 0 , sizeof(tag));
@@ -342,14 +305,14 @@ bool cVNSIData::GetEPGForChannel(ADDON_HANDLE handle, const PVR_CHANNEL &channel
     tag.strWriter           = "";
     tag.iYear               = 0;
     tag.strIMDBNumber       = "";
+    if (tag.strPlotOutline)
+      tag.strEpisodeName    = strdup(tag.strPlotOutline);
+    tag.iFlags              = EPG_TAG_FLAG_UNDEFINED;
 
     PVR->TransferEpgEntry(handle, &tag);
-    delete[] tag.strTitle;
-    delete[] tag.strPlotOutline;
-    delete[] tag.strPlot;
+    free((void*)tag.strEpisodeName);
   }
 
-  delete vresp;
   return true;
 }
 
@@ -359,13 +322,9 @@ bool cVNSIData::GetEPGForChannel(ADDON_HANDLE handle, const PVR_CHANNEL &channel
 int cVNSIData::GetTimersCount()
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_TIMER_GETCOUNT))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return -1;
-  }
+  vrp.init(VNSI_TIMER_GETCOUNT);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
@@ -374,7 +333,6 @@ int cVNSIData::GetTimersCount()
 
   uint32_t count = vresp->extract_U32();
 
-  delete vresp;
   return count;
 }
 
@@ -382,33 +340,28 @@ PVR_ERROR cVNSIData::GetTimerInfo(unsigned int timernumber, PVR_TIMER &tag)
 {
   cRequestPacket vrp;
   memset(&tag, 0, sizeof(tag));
-  if (!vrp.init(VNSI_TIMER_GET))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return PVR_ERROR_UNKNOWN;
-  }
+  vrp.init(VNSI_TIMER_GET);
+  vrp.add_U32(timernumber);
 
-  if (!vrp.add_U32(timernumber))
-    return PVR_ERROR_UNKNOWN;
-
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
-    delete vresp;
     return PVR_ERROR_UNKNOWN;
   }
 
   uint32_t returnCode = vresp->extract_U32();
   if (returnCode != VNSI_RET_OK)
   {
-    delete vresp;
     if (returnCode == VNSI_RET_DATAUNKNOWN)
       return PVR_ERROR_FAILED;
     else if (returnCode == VNSI_RET_ERROR)
       return PVR_ERROR_SERVER_ERROR;
   }
 
+  /* TODO: Implement own timer types to get support for the timer features introduced with PVR API 1.9.7 */
+  tag.iTimerType = PVR_TIMER_TYPE_NONE;
+
   tag.iClientIndex      = vresp->extract_U32();
   int iActive           = vresp->extract_U32();
   int iRecording        = vresp->extract_U32();
@@ -427,28 +380,20 @@ PVR_ERROR cVNSIData::GetTimerInfo(unsigned int timernumber, PVR_TIMER &tag)
   tag.endTime           = vresp->extract_U32();
   tag.firstDay          = vresp->extract_U32();
   tag.iWeekdays         = vresp->extract_U32();
-  tag.bIsRepeating      = tag.iWeekdays == 0 ? false : true;
   char *strTitle = vresp->extract_String();
   strncpy(tag.strTitle, strTitle, sizeof(tag.strTitle) - 1);
-  delete[] strTitle;
 
-  delete vresp;
   return PVR_ERROR_NO_ERROR;
 }
 
 bool cVNSIData::GetTimersList(ADDON_HANDLE handle)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_TIMER_GETLIST))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return false;
-  }
+  vrp.init(VNSI_TIMER_GETLIST);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
-    delete vresp;
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
     return false;
   }
@@ -456,10 +401,14 @@ bool cVNSIData::GetTimersList(ADDON_HANDLE handle)
   uint32_t numTimers = vresp->extract_U32();
   if (numTimers > 0)
   {
-    while (!vresp->end())
+    while (vresp->getRemainingLength() >= 12 * 4 + 1)
     {
       PVR_TIMER tag;
       memset(&tag, 0, sizeof(tag));
+
+      /* TODO: Implement own timer types to get support for the timer features introduced with PVR API 1.9.7 */
+      tag.iTimerType = PVR_TIMER_TYPE_NONE;
+
       tag.iClientIndex      = vresp->extract_U32();
       int iActive           = vresp->extract_U32();
       int iRecording        = vresp->extract_U32();
@@ -478,60 +427,70 @@ bool cVNSIData::GetTimersList(ADDON_HANDLE handle)
       tag.endTime           = vresp->extract_U32();
       tag.firstDay          = vresp->extract_U32();
       tag.iWeekdays         = vresp->extract_U32();
-      tag.bIsRepeating      = tag.iWeekdays == 0 ? false : true;
       char *strTitle = vresp->extract_String();
       strncpy(tag.strTitle, strTitle, sizeof(tag.strTitle) - 1);
       tag.iMarginStart      = 0;
       tag.iMarginEnd        = 0;
 
       PVR->TransferTimerEntry(handle, &tag);
-
-      delete[] strTitle;
     }
   }
-  delete vresp;
   return true;
 }
 
 PVR_ERROR cVNSIData::AddTimer(const PVR_TIMER &timerinfo)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_TIMER_ADD))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return PVR_ERROR_UNKNOWN;
-  }
+  vrp.init(VNSI_TIMER_ADD);
 
   // add directory in front of the title
   std::string path;
-  if(strlen(timerinfo.strDirectory) > 0) {
+  if (strlen(timerinfo.strDirectory) > 0)
+  {
     path += timerinfo.strDirectory;
-    if(path == "/") {
+    if (path == "/")
+    {
       path.clear();
     }
-    else if(path.size() > 1) {
-      if(path[0] == '/') {
+    else if (path.size() > 1)
+    {
+      if (path[0] == '/')
+      {
         path = path.substr(1);
       }
     }
 
-    if(path.size() > 0 && path[path.size()-1] != '/') {
+    if (path.size() > 0 && path[path.size()-1] != '/')
+    {
       path += "/";
     }
   }
 
   // replace directory separators
-  for(std::size_t i=0; i<path.size(); i++) {
-    if(path[i] == '/' || path[i] == '\\') {
+  for (std::size_t i=0; i<path.size(); i++)
+  {
+    if (path[i] == '/' || path[i] == '\\')
+    {
       path[i] = '~';
     }
   }
 
-  if(strlen(timerinfo.strTitle) > 0) {
+  if (strlen(timerinfo.strTitle) > 0)
+  {
     path += timerinfo.strTitle;
   }
 
-  if(path.empty()) {
+  // replace colons
+  for (std::size_t i=0; i<path.size(); i++)
+  {
+    if (path[i] == ':')
+    {
+      path[i] = '|';
+    }
+  }
+
+  if (path.empty())
+  {
     XBMC->Log(LOG_ERROR, "%s - Empty filename !", __FUNCTION__);
     return PVR_ERROR_UNKNOWN;
   }
@@ -540,26 +499,24 @@ PVR_ERROR cVNSIData::AddTimer(const PVR_TIMER &timerinfo)
   uint32_t starttime = timerinfo.startTime - timerinfo.iMarginStart*60;
   uint32_t endtime = timerinfo.endTime + timerinfo.iMarginEnd*60;
 
-  if (!vrp.add_U32(timerinfo.state == PVR_TIMER_STATE_SCHEDULED))     return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(timerinfo.iPriority))   return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(timerinfo.iLifetime))   return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(timerinfo.iClientChannelUid)) return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(starttime))  return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(endtime))    return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(timerinfo.bIsRepeating ? timerinfo.firstDay : 0))   return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(timerinfo.iWeekdays))return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_String(path.c_str()))      return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_String(""))                return PVR_ERROR_UNKNOWN;
+  vrp.add_U32(timerinfo.state == PVR_TIMER_STATE_SCHEDULED);
+  vrp.add_U32(timerinfo.iPriority);
+  vrp.add_U32(timerinfo.iLifetime);
+  vrp.add_U32(timerinfo.iClientChannelUid);
+  vrp.add_U32(starttime);
+  vrp.add_U32(endtime);
+  vrp.add_U32(timerinfo.iWeekdays != PVR_WEEKDAY_NONE ? timerinfo.firstDay : 0);
+  vrp.add_U32(timerinfo.iWeekdays);
+  vrp.add_String(path.c_str());
+  vrp.add_String("");
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (vresp == NULL || vresp->noResponse())
   {
-    delete vresp;
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
     return PVR_ERROR_UNKNOWN;
   }
   uint32_t returnCode = vresp->extract_U32();
-  delete vresp;
   if (returnCode == VNSI_RET_DATALOCKED)
     return PVR_ERROR_ALREADY_PRESENT;
   else if (returnCode == VNSI_RET_DATAINVALID)
@@ -573,23 +530,16 @@ PVR_ERROR cVNSIData::AddTimer(const PVR_TIMER &timerinfo)
 PVR_ERROR cVNSIData::DeleteTimer(const PVR_TIMER &timerinfo, bool force)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_TIMER_DELETE))
-    return PVR_ERROR_UNKNOWN;
+  vrp.init(VNSI_TIMER_DELETE);
+  vrp.add_U32(timerinfo.iClientIndex);
+  vrp.add_U32(force);
 
-  if (!vrp.add_U32(timerinfo.iClientIndex))
-    return PVR_ERROR_UNKNOWN;
-
-  if (!vrp.add_U32(force))
-    return PVR_ERROR_UNKNOWN;
-
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (vresp == NULL || vresp->noResponse())
   {
-    delete vresp;
     return PVR_ERROR_UNKNOWN;
   }
   uint32_t returnCode = vresp->extract_U32();
-  delete vresp;
 
   if (returnCode == VNSI_RET_DATALOCKED)
     return PVR_ERROR_FAILED;
@@ -621,27 +571,25 @@ PVR_ERROR cVNSIData::UpdateTimer(const PVR_TIMER &timerinfo)
   uint32_t endtime = timerinfo.endTime + timerinfo.iMarginEnd*60;
 
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_TIMER_UPDATE))        return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(timerinfo.iClientIndex))      return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(timerinfo.state == PVR_TIMER_STATE_SCHEDULED))     return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(timerinfo.iPriority))   return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(timerinfo.iLifetime))   return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(timerinfo.iClientChannelUid)) return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(starttime))  return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(endtime))    return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(timerinfo.bIsRepeating ? timerinfo.firstDay : 0))   return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_U32(timerinfo.iWeekdays))return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_String(timerinfo.strTitle))   return PVR_ERROR_UNKNOWN;
-  if (!vrp.add_String(""))                return PVR_ERROR_UNKNOWN;
+  vrp.init(VNSI_TIMER_UPDATE);
+  vrp.add_U32(timerinfo.iClientIndex);
+  vrp.add_U32(timerinfo.state == PVR_TIMER_STATE_SCHEDULED);
+  vrp.add_U32(timerinfo.iPriority);
+  vrp.add_U32(timerinfo.iLifetime);
+  vrp.add_U32(timerinfo.iClientChannelUid);
+  vrp.add_U32(starttime);
+  vrp.add_U32(endtime);
+  vrp.add_U32(timerinfo.iWeekdays != PVR_WEEKDAY_NONE ? timerinfo.firstDay : 0);
+  vrp.add_U32(timerinfo.iWeekdays);
+  vrp.add_String(timerinfo.strTitle);
+  vrp.add_String("");
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (vresp == NULL || vresp->noResponse())
   {
-    delete vresp;
     return PVR_ERROR_UNKNOWN;
   }
   uint32_t returnCode = vresp->extract_U32();
-  delete vresp;
   if (returnCode == VNSI_RET_DATAUNKNOWN)
     return PVR_ERROR_FAILED;
   else if (returnCode == VNSI_RET_DATAINVALID)
@@ -655,13 +603,9 @@ PVR_ERROR cVNSIData::UpdateTimer(const PVR_TIMER &timerinfo)
 int cVNSIData::GetRecordingsCount()
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_RECORDINGS_GETCOUNT))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return -1;
-  }
+  vrp.init(VNSI_RECORDINGS_GETCOUNT);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
@@ -669,21 +613,15 @@ int cVNSIData::GetRecordingsCount()
   }
 
   uint32_t count = vresp->extract_U32();
-
-  delete vresp;
   return count;
 }
 
 PVR_ERROR cVNSIData::GetRecordingsList(ADDON_HANDLE handle)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_RECORDINGS_GETLIST))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return PVR_ERROR_UNKNOWN;
-  }
+  vrp.init(VNSI_RECORDINGS_GETLIST);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
@@ -691,7 +629,7 @@ PVR_ERROR cVNSIData::GetRecordingsList(ADDON_HANDLE handle)
   }
 
   CStdString strRecordingId;
-  while (!vresp->end())
+  while (vresp->getRemainingLength() >= 5 * 4 + 5)
   {
     PVR_RECORDING tag;
     memset(&tag, 0, sizeof(tag));
@@ -720,46 +658,30 @@ PVR_ERROR cVNSIData::GetRecordingsList(ADDON_HANDLE handle)
     strncpy(tag.strRecordingId, strRecordingId.c_str(), sizeof(tag.strRecordingId) - 1);
 
     PVR->TransferRecordingEntry(handle, &tag);
-
-    delete[] strChannelName;
-    delete[] strTitle;
-    delete[] strPlotOutline;
-    delete[] strPlot;
-    delete[] strDirectory;
   }
 
-  delete vresp;
-
   return PVR_ERROR_NO_ERROR;
 }
 
 PVR_ERROR cVNSIData::RenameRecording(const PVR_RECORDING& recinfo, const char* newname)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_RECORDINGS_RENAME))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return PVR_ERROR_UNKNOWN;
-  }
+  vrp.init(VNSI_RECORDINGS_RENAME);
 
   // add uid
   XBMC->Log(LOG_DEBUG, "%s - uid: %s", __FUNCTION__, recinfo.strRecordingId);
-  if (!vrp.add_U32(atoi(recinfo.strRecordingId)))
-    return PVR_ERROR_UNKNOWN;
+  vrp.add_U32(atoi(recinfo.strRecordingId));
 
   // add new title
-  if (!vrp.add_String(newname))
-    return PVR_ERROR_UNKNOWN;
+  vrp.add_String(newname);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (vresp == NULL || vresp->noResponse())
   {
-    delete vresp;
     return PVR_ERROR_SERVER_ERROR;
   }
 
   uint32_t returnCode = vresp->extract_U32();
-  delete vresp;
 
   if(returnCode != 0)
    return PVR_ERROR_FAILED;
@@ -770,24 +692,16 @@ PVR_ERROR cVNSIData::RenameRecording(const PVR_RECORDING& recinfo, const char* n
 PVR_ERROR cVNSIData::DeleteRecording(const PVR_RECORDING& recinfo)
 {
   cRequestPacket vrp;
-  if (!vrp.init(recinfo.bIsDeleted ? VNSI_RECORDINGS_DELETED_DELETE : VNSI_RECORDINGS_DELETE))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return PVR_ERROR_UNKNOWN;
-  }
+  vrp.init(recinfo.bIsDeleted ? VNSI_RECORDINGS_DELETED_DELETE : VNSI_RECORDINGS_DELETE);
+  vrp.add_U32(atoi(recinfo.strRecordingId));
 
-  if (!vrp.add_U32(atoi(recinfo.strRecordingId)))
-    return PVR_ERROR_UNKNOWN;
-
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (vresp == NULL || vresp->noResponse())
   {
-    delete vresp;
     return PVR_ERROR_UNKNOWN;
   }
 
   uint32_t returnCode = vresp->extract_U32();
-  delete vresp;
 
   switch(returnCode)
   {
@@ -810,24 +724,18 @@ PVR_ERROR cVNSIData::DeleteRecording(const PVR_RECORDING& recinfo)
 PVR_ERROR cVNSIData::GetRecordingEdl(const PVR_RECORDING& recinfo, PVR_EDL_ENTRY edl[], int *size)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_RECORDINGS_GETEDL))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return PVR_ERROR_UNKNOWN;
-  }
+  vrp.init(VNSI_RECORDINGS_GETEDL);
+  vrp.add_U32(atoi(recinfo.strRecordingId));
 
-  if (!vrp.add_U32(atoi(recinfo.strRecordingId)))
-    return PVR_ERROR_UNKNOWN;
-
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (vresp == NULL || vresp->noResponse())
   {
-    delete vresp;
     return PVR_ERROR_UNKNOWN;
   }
 
   *size = 0;
-  while (!vresp->end() && *size < PVR_ADDON_EDL_LENGTH)
+  while (vresp->getRemainingLength() >= 2 * 8 + 4 &&
+         *size < PVR_ADDON_EDL_LENGTH)
   {
     edl[*size].start = vresp->extract_S64();
     edl[*size].end = vresp->extract_S64();
@@ -835,21 +743,15 @@ PVR_ERROR cVNSIData::GetRecordingEdl(const PVR_RECORDING& recinfo, PVR_EDL_ENTRY
     (*size)++;
   }
 
-  delete vresp;
-
   return PVR_ERROR_NO_ERROR;
 }
 
 int cVNSIData::GetDeletedRecordingsCount()
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_RECORDINGS_DELETED_GETCOUNT))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return -1;
-  }
+  vrp.init(VNSI_RECORDINGS_DELETED_GETCOUNT);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
@@ -857,21 +759,15 @@ int cVNSIData::GetDeletedRecordingsCount()
   }
 
   uint32_t count = vresp->extract_U32();
-
-  delete vresp;
   return count;
 }
 
 PVR_ERROR cVNSIData::GetDeletedRecordingsList(ADDON_HANDLE handle)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_RECORDINGS_DELETED_GETLIST))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return PVR_ERROR_UNKNOWN;
-  }
+  vrp.init(VNSI_RECORDINGS_DELETED_GETLIST);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
     XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
@@ -879,7 +775,7 @@ PVR_ERROR cVNSIData::GetDeletedRecordingsList(ADDON_HANDLE handle)
   }
 
   CStdString strRecordingId;
-  while (!vresp->end())
+  while (vresp->getRemainingLength() >= 5 * 4 + 5)
   {
     PVR_RECORDING tag;
     memset(&tag, 0, sizeof(tag));
@@ -908,41 +804,24 @@ PVR_ERROR cVNSIData::GetDeletedRecordingsList(ADDON_HANDLE handle)
     strncpy(tag.strRecordingId, strRecordingId.c_str(), sizeof(tag.strRecordingId) - 1);
 
     PVR->TransferRecordingEntry(handle, &tag);
-
-    delete[] strChannelName;
-    delete[] strTitle;
-    delete[] strPlotOutline;
-    delete[] strPlot;
-    delete[] strDirectory;
   }
 
-  delete vresp;
-
   return PVR_ERROR_NO_ERROR;
 }
 
 PVR_ERROR cVNSIData::UndeleteRecording(const PVR_RECORDING& recinfo)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_RECORDINGS_DELETED_UNDELETE))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return PVR_ERROR_UNKNOWN;
-  }
+  vrp.init(VNSI_RECORDINGS_DELETED_UNDELETE);
+  vrp.add_U32(atoi(recinfo.strRecordingId));
 
-  if (!vrp.add_U32(atoi(recinfo.strRecordingId)))
-    return PVR_ERROR_UNKNOWN;
-
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (vresp == NULL || vresp->noResponse())
   {
-    delete vresp;
     return PVR_ERROR_UNKNOWN;
   }
 
   uint32_t returnCode = vresp->extract_U32();
-  delete vresp;
-
   switch(returnCode)
   {
     case VNSI_RET_DATALOCKED:
@@ -964,22 +843,15 @@ PVR_ERROR cVNSIData::UndeleteRecording(const PVR_RECORDING& recinfo)
 PVR_ERROR cVNSIData::DeleteAllRecordingsFromTrash()
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_RECORDINGS_DELETED_DELETE_ALL))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return PVR_ERROR_UNKNOWN;
-  }
+  vrp.init(VNSI_RECORDINGS_DELETED_DELETE_ALL);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (vresp == NULL || vresp->noResponse())
   {
-    delete vresp;
     return PVR_ERROR_UNKNOWN;
   }
 
   uint32_t returnCode = vresp->extract_U32();
-  delete vresp;
-
   switch(returnCode)
   {
     case VNSI_RET_DATALOCKED:
@@ -1005,7 +877,7 @@ bool cVNSIData::OnResponsePacket(cResponsePacket* pkt)
 
 void *cVNSIData::Process()
 {
-  cResponsePacket* vresp;
+  std::unique_ptr<cResponsePacket> vresp;
 
   while (!IsStopped())
   {
@@ -1026,17 +898,7 @@ void *cVNSIData::Process()
     // CHANNEL_REQUEST_RESPONSE
     if (vresp->getChannelID() == VNSI_CHANNEL_REQUEST_RESPONSE)
     {
-      CLockObject lock(m_mutex);
-      SMessages::iterator it = m_queue.find(vresp->getRequestID());
-      if (it != m_queue.end())
-      {
-        it->second.pkt = vresp;
-        it->second.event->Broadcast();
-      }
-      else
-      {
-        delete vresp;
-      }
+      m_queue.Set(std::move(vresp));
     }
 
     // CHANNEL_STATUS
@@ -1061,7 +923,6 @@ void *cVNSIData::Process()
         else
           XBMC->QueueNotification(QUEUE_INFO, strMessageTranslated);
 
-        delete[] msgstr;
         if (g_bCharsetConv)
           XBMC->FreeString(strMessageTranslated);
       }
@@ -1074,9 +935,6 @@ void *cVNSIData::Process()
 
 //        PVR->Recording(str1, str2, on!=0?true:false);
         PVR->TriggerTimerUpdate();
-
-        delete[] str1;
-        delete[] str2;
       }
       else if (vresp->getRequestID() == VNSI_STATUS_TIMERCHANGE)
       {
@@ -1099,16 +957,13 @@ void *cVNSIData::Process()
         XBMC->Log(LOG_DEBUG, "Server requested Epg update for channel: %d", channel);
         PVR->TriggerEpgUpdate(channel);
       }
-
-      delete vresp;
     }
 
     // UNKOWN CHANNELID
 
-    else if (!OnResponsePacket(vresp))
+    else if (!OnResponsePacket(vresp.get()))
     {
       XBMC->Log(LOG_ERROR, "%s - Rxd a response packet on channel %lu !!", __FUNCTION__, vresp->getChannelID());
-      delete vresp;
     }
   }
   return NULL;
@@ -1117,49 +972,33 @@ void *cVNSIData::Process()
 int cVNSIData::GetChannelGroupCount(bool automatic)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_CHANNELGROUP_GETCOUNT))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return 0;
-  }
+  vrp.init(VNSI_CHANNELGROUP_GETCOUNT);
+  vrp.add_U32(automatic);
 
-  if (!vrp.add_U32(automatic))
-  {
-    return 0;
-  }
-
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (vresp == NULL || vresp->noResponse())
   {
-    delete vresp;
     return 0;
   }
 
   uint32_t count = vresp->extract_U32();
 
-  delete vresp;
   return count;
 }
 
 bool cVNSIData::GetChannelGroupList(ADDON_HANDLE handle, bool bRadio)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_CHANNELGROUP_LIST))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return false;
-  }
-
+  vrp.init(VNSI_CHANNELGROUP_LIST);
   vrp.add_U8(bRadio);
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (vresp == NULL || vresp->noResponse())
   {
-    delete vresp;
     return false;
   }
 
-  while (!vresp->end())
+  while (vresp->getRemainingLength() >= 1 + 1)
   {
     PVR_CHANNEL_GROUP tag;
     memset(&tag, 0, sizeof(tag));
@@ -1170,35 +1009,26 @@ bool cVNSIData::GetChannelGroupList(ADDON_HANDLE handle, bool bRadio)
     tag.iPosition = 0;
 
     PVR->TransferChannelGroup(handle, &tag);
-
-    delete[] strGroupName;
   }
 
-  delete vresp;
   return true;
 }
 
 bool cVNSIData::GetChannelGroupMembers(ADDON_HANDLE handle, const PVR_CHANNEL_GROUP &group)
 {
   cRequestPacket vrp;
-  if (!vrp.init(VNSI_CHANNELGROUP_MEMBERS))
-  {
-    XBMC->Log(LOG_ERROR, "%s - Can't init cRequestPacket", __FUNCTION__);
-    return false;
-  }
-
+  vrp.init(VNSI_CHANNELGROUP_MEMBERS);
   vrp.add_String(group.strGroupName);
   vrp.add_U8(group.bIsRadio);
   vrp.add_U8(1); // filter channels
 
-  cResponsePacket* vresp = ReadResult(&vrp);
+  auto vresp = ReadResult(&vrp);
   if (vresp == NULL || vresp->noResponse())
   {
-    delete vresp;
     return false;
   }
 
-  while (!vresp->end())
+  while (vresp->getRemainingLength() >= 2 * 4)
   {
     PVR_CHANNEL_GROUP_MEMBER tag;
     memset(&tag, 0, sizeof(tag));
@@ -1210,6 +1040,5 @@ bool cVNSIData::GetChannelGroupMembers(ADDON_HANDLE handle, const PVR_CHANNEL_GR
     PVR->TransferChannelGroupMember(handle, &tag);
   }
 
-  delete vresp;
   return true;
 }
